/*
 * Copyright (c) 2017, 2018, Oracle and/or its affiliates.
 * Copyright (c) 2013, Regents of the University of California
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other materials provided
 * with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.oracle.graal.python.test.builtin;

import java.nio.file.*;

import org.junit.*;

import com.oracle.graal.python.test.PythonTests;

import static com.oracle.graal.python.test.PythonTests.*;

public class ImportTests {
    @Before
    public void setup() {
        PythonTests.enterContext();
    }

    @Test
    public void relativeImportTest() {
        Path script = Paths.get("relative_import.py");
        assertPrints("module Y\n" + //
                        "cos(100) = 0.8623188722876839\n" + //
                        "module X\n" + //
                        "module Z\n" + //
                        "module A\n" + //
                        "module B\n" + //
                        "module C\n" + //
                        "after importing moduleY\n", script);
    }

    @Test
    public void importStandardLib() {
        String source = "import bisect\n" + //
                        "bisect.foo = 42\n" + //
                        "print(bisect.foo)\n";
        assertPrints("42\n", source);
    }

    @Test
    public void module__file__() {
        String source = "import bisect\n" + //
                        "print(bisect.__file__)\n";
        assertPrintContains("bisect.py\n", source);
    }

    @Test
    public void module__file__1() {
        String source = "import __future__\n" + //
                        "print(__future__.__file__)\n";
        assertPrintContains("__future__.py\n", source);
    }
}
