/*
 * Copyright (c) 2017, 2018, Oracle and/or its affiliates.
 * Copyright (c) 2013, Regents of the University of California
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other materials provided
 * with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.oracle.graal.python.test.module;

import org.junit.*;

import com.oracle.graal.python.test.PythonTests;

import static com.oracle.graal.python.test.PythonTests.*;

public class MathTests {
    @Before
    public void setUp() {
        PythonTests.enterContext();
    }

    @Test
    public void piAndE() {
        String source = "import math\n" + //
                        "print(math.e)\n" + //
                        "print(math.pi)\n" + //
                        "print(math.e + math.pi)\n";
        assertPrints("2.718281828459045\n" + "3.141592653589793\n" + "5.859874482048838\n", source);
    }

    @Test
    public void logAndExp() {
        String source = "import math\n" + //
                        "a = math.log(1 + math.e)\n" + //
                        "b = 10\n" + //
                        "c = math.exp(b)\n" + //
                        "print(a // c)\n";
        assertPrints("0.0\n", source);
    }

    @Test
    public void sinAndCos() {
        String source = "import math\n" + //
                        "x = math.pi\n" + //
                        "y = math.sin(x)\n" + //
                        "print(y // math.cos(math.e))\n";
        assertPrints("-1.0\n", source);
    }

}
