/*
 * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.cext;

import com.oracle.graal.python.PythonLanguage;
import com.oracle.graal.python.builtins.objects.cext.DynamicObjectNativeWrapper.PrimitiveNativeWrapper;

public abstract class CApiGuards {

    public static boolean isPrimitiveNativeWrapper(Object object) {
        return object instanceof PrimitiveNativeWrapper;
    }

    public static boolean isNativeWrapper(Object object) {
        return object instanceof PythonNativeWrapper;
    }

    public static boolean isNativeNull(Object object) {
        return object instanceof PythonNativeNull;
    }

    public static boolean isSpecialSingleton(Object delegate) {
        return PythonLanguage.getSingletonNativeWrapperIdx(delegate) != -1;
    }

    /**
     * This guard defines the range of integer values for which PInt objects (and in our particular
     * case, native wrappers) are cached.
     */
    public static boolean isSmallInteger(int i) {
        return -5 <= i && i < 257;
    }

    /** see {@link #isSmallInteger(int)} */
    public static boolean isSmallLong(long i) {
        return -5 <= i && i < 257;
    }

    public static boolean isSmallIntegerWrapper(PrimitiveNativeWrapper nativeWrapper) {
        return nativeWrapper.isIntLike() && isSmallLong(nativeWrapper.getLong());
    }
}
